﻿/* Copyright 2018, Kasyan Servetsky
January 6, 2018
Written by Kasyan Servetsky
http://www.kasyan.ho.com.ua
e-mail: askoldich@yahoo.com */
//=======================================================================================
var scriptName = "Package for print",
calledFromBatchProcessor = (typeof arguments != "undefined" && CalledFromBatchProcessor()) ? true : false;

if (calledFromBatchProcessor) { // called as a 'secondary' from 
	var args = NormalizeArgs(arguments);
	Main();
}
else { // called as a 'regular' script
	PreCheck();
}

//===================================== FUNCTIONS =======================================
function Main() {
	try { // if something goes wrong in the try-catch block, the batch processor won't stop here. It will log the error message and continue further
		var result,  // if packaged successfully, returns true, otherwise - false
		link, font, // declare some variables at start of the function
		doc = app.activeDocument, // The frontmost document
		appVersion = Number(String(app.version).split(".")[0]), // the InDesign version number
		docBasicName = doc.name.replace(/\.indd$/, ""); // remove the extension from the document's name

		// Parameters of the 'packageForPrint' method =================================================		
		if (calledFromBatchProcessor && args.length >= 8) { // get parameters from the arguments file
			var packageAllDocsFolderPath = (typeof args[0] == "undefined" || args[0] == null  || args[0] == "") ? doc.filePath.absoluteURI + "/Archive/" : args[0], // the path to the folder in which to place the packaged files.
			copyingFonts = args[1], // copyingFonts - Boolean - If true, copies fonts used in the document to the package folder.
			copyingLinkedGraphics = args[2], // copyingLinkedGraphics - Boolean - If true, copies linked graphics files to the package folder. 
			copyingProfiles = args[3], // copyingProfiles - Boolean - If true, copies color profiles to the package folder. 
			updatingGraphics = args[4], // updatingGraphics - Boolean - If true, updates graphics links to the package folder. 
			includingHiddenLayers = args[5], // [Unavailable in CS3, for some reason] includingHiddenLayers - Boolean - If true, copies fonts and links from hidden layers to the package. 
			ignorePreflightErrors = args[6], // ignorePreflightErrors - Boolean - If true, ignores preflight errors and proceeds with the packaging. If false, cancels the packaging when errors exist. 
			creatingReport = args[7], // creatingReport - Boolean - If true, creates a package report that includes printing instructions, print settings, lists of fonts, links and required inks, and other information. 
			includeIdml = args[8], // [Only CC 2014 (ver.10) and above] includeIdml - Boolean - If true, generates and includes IDML in the package folder. (Optional)
			includePdf = args[9], // [Only CC 2014 (ver.10) and above] includePdf - Boolean - If true, generates and includes PDF in the package folder. (Optional)
			pdfStyle = args[10], // [Only CC 2014 (ver.10) and above] pdfStyle - String - If specified and PDF is to be included, use this style for PDF export if it is valid, otherwise use the last used PDF preset. (Optional)
			useDocumentHyphenationExceptionsOnly = args[11], // [Only CC 2018 (ver.13) and above] useDocumentHyphenationExceptionsOnly - Boolean - If this option is selected, InDesign flags this document so that it does not reflow when someone else opens or edits it on a computer that has different hyphenation and dictionary settings. (Optional)
			versionComments = args[12], // [Only CS3 (ver. 5) and above] versionComments - String - The comments for the version. (Optional) 
			forceSave = args[13]; // [Only CS3 (ver. 5) and above] forceSave - Boolean - If true, forcibly saves a version. (Optional) (default: false) 
		}
		else { // no arguments file is selected, or run as a regular script -- use the default  'hardcoded' settings below
			var packageAllDocsFolderPath = doc.filePath.absoluteURI + "/Archive/", // the path to the folder where the active document is located
			copyingFonts = false, // copyingFonts - Boolean - If true, copies fonts used in the document to the package folder.
			copyingLinkedGraphics = true, // copyingLinkedGraphics - Boolean - If true, copies linked graphics files to the package folder. 
			copyingProfiles = false, // copyingProfiles - Boolean - If true, copies color profiles to the package folder. 
			updatingGraphics = true, // updatingGraphics - Boolean - If true, updates graphics links to the package folder. 
			includingHiddenLayers = true, // [Unavailable in CS3, for some reason] includingHiddenLayers - Boolean - If true, copies fonts and links from hidden layers to the package. 
			ignorePreflightErrors = true, // ignorePreflightErrors - Boolean - If true, ignores preflight errors and proceeds with the packaging. If false, cancels the packaging when errors exist. 
			creatingReport = false, // creatingReport - Boolean - If true, creates a package report that includes printing instructions, print settings, lists of fonts, links and required inks, and other information. 
			includeIdml = true, // [Only CC 2014 (ver.10) and above] includeIdml - Boolean - If true, generates and includes IDML in the package folder. (Optional)
			includePdf = true, // [Only CC 2014 (ver.10) and above] includePdf - Boolean - If true, generates and includes PDF in the package folder. (Optional)
			pdfStyle = "[High Quality Print]", // [Only CC 2014 (ver.10) and above] pdfStyle - String - If specified and PDF is to be included, use this style for PDF export if it is valid, otherwise use the last used PDF preset. (Optional)
			useDocumentHyphenationExceptionsOnly = false, // [Only CC 2018 (ver.13) and above] useDocumentHyphenationExceptionsOnly - Boolean - If this option is selected, InDesign flags this document so that it does not reflow when someone else opens or edits it on a computer that has different hyphenation and dictionary settings. (Optional)
			versionComments = undefined, // [Only CS3 (ver. 5) and above] versionComments - String - The comments for the version. (Optional) 
			forceSave = undefined; // [Only CS3 (ver. 5) and above] forceSave - Boolean - If true, forcibly saves a version. (Optional) (default: false) 
		}
		// ====================================================================================

		if (!calledFromBatchProcessor) { // only when called as a regular script
			// Only for CC 2014 and above. If 'Include PDF(Print)' is selected, but the PDF preset specified in the parameters is unavailable,
			// in this case the last used PDF preset will be used instead.
			if (appVersion >= 10 && includePdf && !app.pdfExportPresets.itemByName(pdfStyle).isValid) {
				var answer = confirm("PROBLEM: the PDF preset '" + pdfStyle + "' specified in the parameters is unavailable so the last used PDF preset will be used instead. Do you want to continue?", false, scriptName); // log a warning about missin PDF preset
				if (!answer) exit();
			}
		}
		
		var packageAllDocsFolder = new Folder(packageAllDocsFolderPath);
		if (!packageAllDocsFolder.exists) packageAllDocsFolder.create(); // create if it doesn't exist yet
		
		var packageDocFolderPath = packageAllDocsFolderPath + docBasicName; // the path to the package folder. Used as the 'to' parameter of the 'packageForPrint' method  - Data Type: File - The folder, alias, or path in which to place the packaged files
		var packageDocFolder = new Folder(packageDocFolderPath); // the reference to the package folder
		
		if (!packageDocFolder.exists) { // create if it doesn't exist yet
			packageDocFolder.create();
		}
		else { // if exists, don't overwrite it: create a new folder with unique incrementin number at the end of the name
			var increment = 1;
			while (packageDocFolder.exists) {
				packageDocFolder = new Folder(packageAllDocsFolderPath  + "/" + docBasicName + "_(" + increment++ + ")");
			}
			packageDocFolder.create();
		}		
		
		if (calledFromBatchProcessor) { // only when called form batch processor
			// Only for CC 2014 and above. If 'Include PDF(Print)' is selected, but the PDF preset specified in the parameters is unavailable,
			// in this case the last used PDF preset will be used instead.
			if (appVersion >= 10 && includePdf && !app.pdfExportPresets.itemByName(pdfStyle).isValid) {
				g.WriteToFile("PROBLEM: the PDF preset '" + pdfStyle + "' specified in the parameters is unavailable so the last used PDF preset will be used instead."); // log a warning about missin PDF preset
			}
			
			// log the list of missing links
			var links = doc.links.everyItem().getElements(); // get all the links
			for (var i = 0; i < links.length; i++) { // loop through all the links
				link = doc.links[i]; // the current link
				if (link.status == LinkStatus.LINK_MISSING) { // if the current link is missing
					g.WriteToFile("PROBLEM: link is missing - " + decodeURI(link.filePath)); // write it's path into the log
				}
			}

			// log the list of missing fonts
			var fonts = doc.fonts.everyItem().getElements(); // get all the fonts
			for (var f = 0; f < fonts.length; f++) { // loop through all the fonts
				font = fonts[f]; // the current font
				if (font.status != FontStatus.INSTALLED) { // if the current font is not installed
//~ 					$.bp();
					g.WriteToFile("PROBLEM: font is not installed - " + font.name.replace(/\t/, " - ")); // write it's name into the log
				}
			}
		}

		if (appVersion >= 13) { // Version CC 2018 and above
			result = doc.packageForPrint(packageDocFolder, copyingFonts, copyingLinkedGraphics, copyingProfiles, updatingGraphics, includingHiddenLayers, ignorePreflightErrors, creatingReport, includeIdml, includePdf, pdfStyle, useDocumentHyphenationExceptionsOnly, versionComments, forceSave);
		}
		else if (appVersion >= 10 && appVersion <= 12) { // Version CC 2014 -- CC 2017
			result = doc.packageForPrint(packageDocFolder, copyingFonts, copyingLinkedGraphics, copyingProfiles, updatingGraphics, includingHiddenLayers, ignorePreflightErrors, creatingReport, includeIdml, includePdf, pdfStyle, versionComments, forceSave);
		}
		else if (appVersion >= 6 && appVersion <= 9) { // Versions CS4 -- CC
			result = doc.packageForPrint(packageDocFolder, copyingFonts, copyingLinkedGraphics, copyingProfiles, updatingGraphics, includingHiddenLayers, ignorePreflightErrors, creatingReport, versionComments, forceSave);
		}
		else if (appVersion == 5) { // Version CS3
			result = doc.packageForPrint(packageDocFolder, copyingFonts, copyingLinkedGraphics, copyingProfiles, updatingGraphics, ignorePreflightErrors, creatingReport, versionComments, forceSave);
		}	
		else if (appVersion >= 3 && appVersion <= 4) { // Versions CS -- CS2
			result = doc.packageForPrint(packageDocFolder, copyingFonts, copyingLinkedGraphics, copyingProfiles, updatingGraphics, includingHiddenLayers, ignorePreflightErrors, creatingReport);
		}

		if (result) { // the result  of packaging the document: success (true)
			if (calledFromBatchProcessor) g.WriteToFile("Successfully packaged the document. (Time: " + GetTime() + ")");
		}
		else { // failed (false)
			if (calledFromBatchProcessor) g.WriteToFile("Unable to package the document: something went wrong.");
		}
	}
	catch(err) { // if an error occurs, catch it and write the  document's name, error message and line# where it happened into the log
		if (calledFromBatchProcessor) g.WriteToFile("ERROR: " + doc.name + " - " + err.message + ", Line: " + err.line + ", Time: " + GetTime());
		$.writeln(err.message + ", line: " + err.line);
	}
}
//=======================================================================================
function GetTime() { // get exact time for the log -- hr : min : sec -- so we could know when exactly an error occured
	var date = new Date();
	var dateString = date.getHours() + ":" + date.getMinutes() + ":" + date.getSeconds();
	return dateString;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function CalledFromBatchProcessor() { // Let's see if it's called as a 'secondary' (returns true) or as a 'regular' (returns false) script
	var result = false;
	
	try {
		var arr = $.stack.split(/[\n]/);
		if (arr.length > 0 && arr[0].match(/\[Batch processor/i) != null) {
			result = true;
		}
	}
	catch(err) {}

	return result;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function NormalizeArgs(args) { // convert a string value read from the arguments file to actual boolen data type (or undefined)
	for (var i = 0; i < args.length; i++) {
		if (args[i] == "true" || args[i] == "false" || args[i] == "undefined" || args[i] == "null") {
			args[i] = eval(args[i]);
		}
	}
	
	return args;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function PreCheck() { // executed only when run as a regular script to check if a document is open and has been saved (isn't a newly created one)
	if (app.documents.length == 0) ErrorExit("Please open a document and try again.", true);
	var doc = app.activeDocument; // used only when run as a regular script to check the active document
	if (doc.converted) ErrorExit("The current document has been modified by being converted from older version of InDesign. Please save the document and try again.", true);
	if (!doc.saved) ErrorExit("The current document has not been saved since it was created. Please save the document and try again.", true);
	Main(); // So far, so good. Let's go further and call the 'Main' function
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function ErrorExit(error, icon) { // Something went totally wrong.
	alert(error, scriptName, icon); // Give a warning
	exit(); // and stop the script
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------